Program NetStats;

{
   Reads a FidoNet NODELIST and outputs a list of stats about the nodelist.
}

Uses
    DOS,
    CRT,
    Printer;

CONST
 Version = 'NetStats 4.0.5 2023/01/16';

{ 2018/02/16 vbc - 3.1.1 Changed date format to UNIX (ccyy/mm/dd) from USA. }
{ 2019/01/10 vbc - 3.1.2 Changed comments regard where to obtain the latest version
                         as original programmer has left Fido }
{ 2022/10/30 vbc - 3.2   Output files now Netstats.rpt (instead of .OUT.
                         Changed where to get it from etc }
{ 2022/10/31 vbc - 3.2.1 Removed any baud rate zero totals from printing.
                         Updated the archive name netstu32.zip }
{ 2022/11/10 vbc - 4.0   Clean up output & add missing reporting fields
                         taken from the old Cobol program.
                         and clean up reporting.  }
{ 2022/11/14 vbc - 4.0.3 Coding to extract node counts + Hold & Down by zone. }
{ 2022/11/21 vbc - 4.0.4 Added Zone names (nodelist field 3) to reports & display }
{ 2023/01/16 vbc - 4.0.5 V32b, V42B not zero omitted and needs this lit after IP (flags), }

TYPE
    Entry     = Record  { these may be useful while working in the pgm }
                  Kind        : string;
                  Number      : String;
                  BbsName     : string;
                  City_ST     : string;
                  Sysop_Name  : string;
                  FonNumbr    : string;
                  BaudRate    : String;
                  Flags       : string;
                end;

    NodeItem  = record
                   Net        : word;
                   Node       : word;
                   FonNumbr   : String[10];
                end;

{ Var   section  }
VAR
   NetNumber : word;
   Buf       : Array[1..20480] of Char;

   Code,
   k, n      : integer;

   { counters ... }

   IfnaNum,
   i, AdminOVHD, NonComments                                          : LongInt;
   ZoneCount, RegionCount, HostCount, HubCount, PvtCount, DownCount   : LongInt;
   HoldCount, Baud33600, Baud9600, Baud2400, Baud1200, Baud300, ISDN  : LongInt;
   CrashMail, MailOnly, HST, PEP, MAX , H16                           : LongInt;
   IBN, IFC, IFT, ITN, IVM, IP, INO4                                  : Longint;
   V32, V32b, V32T, V34, V42, V42b                                    : LongInt;
   XA, XB, XC, XP, XR, XW, XX, Zone                                   : LongInt;

 { New for v4.1  }
   ActivSysops,
   Z1Down, Z2Down, Z3Down, Z4Down                                     : longint;
   Z1Hold, Z2Hold, Z3Hold, Z4Hold                                     : longint;
   Z1Pvt,  Z2Pvt,  Z3Pvt,  Z4Pvt                                      : longint;
   Z1Sysops, Z2Sysops, Z3Sysops, Z4Sysops                             : longint;
   Z1Region, Z2Region, Z3Region, Z4Region                             : longint;
   Z1Host, Z2Host, Z3Host, Z4Host                                     : Longint;
   Z1Hub, Z2Hub, Z3Hub, Z4Hub                                         : Longint;

   Z1ZoneName : string;
   Z2ZoneName : string;
   Z3ZoneName : string;
   Z4ZoneName : string;

   ParmCount  : byte;

   { handles.....}
   OutF       : text;    { will be the output file handle }
   INodelist  : text;    { St. Louis format nodelist file }

   { file name strings ....}
   IfnaList   : string;
   OutFile    : string;

   ALine      : string;

   Parts      : array[1..8] of string;

   AComment   : Boolean;

   Lines      : Entry;

   Apoint     : NodeItem;

   FTIME      : DateTime;

   FYear,FMonth,FDay :Integer;
   FKb : real;

{ Label section  }
LABEL
    GetFileNames,
    GotFileNames;

{ ------ Procedure Definitions ------- }
Function PERCENTS (Num1,Num2:longint) :Real;
Begin
  Percents := 100*(num1/num2);
End;

PROCEDURE ParseLines(OneLine:string; var LineParts: Entry ;Var Comment:Boolean);
Var
   i      : integer;
   k      : integer;
   c      : string[1];
   Pieces : array[1..8] of string;

begin
     Pieces[1] := '';
     Pieces[2] := '';
     Pieces[3] := '';
     Pieces[4] := '';
     Pieces[5] := '';
     Pieces[6] := '';
     Pieces[7] := '';
     Pieces[8] := '';

     LineParts.Kind        := 'xxx';
     LineParts.Number      := '';
     LineParts.BbsName     := '';
     LineParts.City_ST     := '';
     LineParts.Sysop_Name  := '';
     LineParts.FonNumbr    := '';
     LineParts.BaudRate    := '';
     LineParts.Flags       := '';


   k := POS(';',OneLine);
   If ( (k > 0) OR ( Length(OneLine) < 2) ) then
   begin
         Comment := True;
         Exit;
   end;

   i := 0;
   k := 1;

   while i < Length(OneLine)  do
   begin
      inc(i);
      if (OneLine[i] <> ',') OR ( k = 8) then
         Pieces[k] := Pieces[k] + OneLine[i]
      else
         inc(k);
   end;

   LineParts.Kind        := Pieces[1];
   LineParts.Number      := Pieces[2];
   LineParts.BbsName     := Pieces[3];
   LineParts.City_ST     := Pieces[4];
   LineParts.Sysop_Name  := Pieces[5];
   LineParts.FonNumbr    := Pieces[6];
   LineParts.BaudRate    := Pieces[7];
   LineParts.Flags       := Pieces[8];

   Comment := False;
end; { of the Procedure ParseLines }
{--------------------------------------------------------------------}

PROCEDURE STDIO;
{ the following allows redirection of std i/o in TP4 or TP5 }
{  WILL DISABLE USE OF COLORS ! }
Begin
        Assign(Input,'');
        Reset (Input);

        Assign (Output,'');
        Rewrite(Output);
End; { of PROCEDURE STDIO }

PROCEDURE StripOut(What: string; VAR Target:string);
Begin
                While Pos(What,Target) > 0 do
                  Target[Pos(What,Target)] := ' ';
End;

Procedure Show_Em(DevName:String);
Var
  i2 :  longInt;

Begin
  i2 := 0;

{ If DevName = '' then
    DevName := 'CON';
 If DevName <> 'CON' then WriteLn(' ');  WriteLn(' Sending output to file/device : ',DevName); }

 WriteLn(' ');  WriteLn(' Sending output to file/device : ',DevName);
 Assign(OutF,DevName);
 ReWrite(OutF);
 WriteLn(OutF,'                    ',Version);
 WriteLn(OutF);
 WriteLn(OutF, ' Input Nodelist  ', IfnaList);
 WriteLn(OutF, ' Date            ',FYear:4,'/',FMonth,'/',FDay );
 WriteLn(OutF, ' File Size       ',FKb:4:2,' KBytes.');
 WriteLn(OutF);
 WriteLn(OutF,' The Nodelist has ',i,' Nodes in it');
 WriteLn(OutF,'  and a total of',NonComments:5,' non-comment entries.');
 WriteLn(OutF);
 WriteLn(OutF,' Includes        ',ZoneCount:5,' Zones');
 WriteLn(OutF,'                 ',RegionCount:5,' Regions');
 WriteLn(OutF,'                 ',HostCount:5, ' Hosts');
 WriteLn(OutF,'                 ',HubCount:5,' Hubs');
 WriteLn(OutF);
 WriteLn(OutF,' Admin Overhead   ',AdminOVHD:4,' (',PERCENTS(AdminOVHD,i):5:2,'%)');
 WriteLn(OutF,'            and  ',PvtCount:5,' private nodes');
 WriteLn(OutF,'                 ',DownCount:5,' nodes down');
 WriteLn(OutF,'                 ' ,HoldCount:5,' nodes on hold');
 WriteLn(OutF);
 i2 := HoldCount+DownCount+PvtCount;
 WriteLn(OutF,' Offline Overhead ',i2:4,' ( ',PERCENTS(i2,i):5:2,'%)');
 WriteLn(OutF);
 WriteLn(OutF,' CrashMail capable',CrashMail:4,' (',PERCENTS(CrashMail ,i):5:2,'%)');
 WriteLn(OutF,' MailOnly Nodes   ',MailOnly:4,' (',PERCENTS(MailOnly ,i):5:2,'%)');
 WriteLn(OutF);
 WriteLn(OutF,' Zone Break Down');
 WriteLn(OutF,' Zone 1 - ', Z1ZoneName);
 WriteLn(OutF,'        Region',Z1Region:4);
 WriteLn(OutF,'        Hosts ',Z1Host:4);
 WriteLn(OutF,'        Hubs  ',Z1Hub:4);
 WriteLn(OutF,'        Sysops',Z1Sysops:4);
 WriteLn(OutF,'        Hold  ',Z1Hold:4);
 WriteLn(OutF,'        Down  ',Z1Down:4);
 WriteLn(OutF,'        Pvt   ',Z1Pvt:4);
 WriteLn(OutF);
 WriteLn(Outf,' Zone 2 - ', Z2ZoneName);
 WriteLn(OutF,'        Region',Z2Region:4);
 WriteLn(OutF,'        Hosts ',Z2Host:4);
 WriteLn(OutF,'        Hubs  ',Z2Hub:4);
 WriteLn(OutF,'        Sysops',Z2Sysops:4);
 WriteLn(OutF,'        Hold  ',Z2Hold:4);
 WriteLn(OutF,'        Down  ',Z2Down:4);
 WriteLn(OutF,'        Pvt   ',Z2Pvt:4);
 WriteLn(OutF);
 WriteLn(Outf,' Zone 3 - ', Z3ZoneName);
 WriteLn(OutF,'        Region',Z3Region:4);
 WriteLn(OutF,'        Hosts ',Z3Host:4);
 WriteLn(OutF,'        Hubs  ',Z3Hub:4);
 WriteLn(OutF,'        Sysops',Z3Sysops:4);
 WriteLn(OutF,'        Hold  ',Z3Hold:4);
 WriteLn(OutF,'        Down  ',Z3Down:4);
 WriteLn(OutF,'        Pvt   ',Z3Pvt:4);
 WriteLn(OutF);
 WriteLn(OutF,' Zone 4 - ', Z4ZoneName);
 WriteLn(OutF,'        Region',Z4Region:4);
 WriteLn(OutF,'        Hosts ',Z4Host:4);
 WriteLn(OutF,'        Hubs  ',Z4Hub:4);
 WriteLn(OutF,'        Sysops',Z4Sysops:4);
 WriteLn(OutF,'        Hold  ',Z4Hold:4);
 WriteLn(OutF,'        Down  ',Z4Down:4);
 WriteLn(OutF,'        Pvt   ',Z4Pvt:4);
 ActivSysops := Z1Sysops + Z2Sysops + Z3Sysops + Z4Sysops;
 WriteLn(OutF);
 WriteLn(OutF,' Total Active Sysops ',ActivSysops,' Excluding Overheads');
 WriteLn(OutF);
 WriteLn(OutF,'The following values include Down, Hold and Private sysops');
 WriteLn(OutF);
 WriteLn(OutF,' Speed Summaries (zero omitted)');
 WriteLn(OutF,' ===============');
 If Baud33600 <> 0 then
    WriteLn(OutF,'          33600 = ',Baud33600:4,' ( ',PERCENTS(Baud33600,i):5:2,'%)');
 If Baud9600 <> 0 then
    WriteLn(OutF,'           9600 = ',Baud9600:4,' ( ',PERCENTS(Baud9600,i):5:2,'%)');
 If H16 <> 0 then
    WriteLn(OutF,'                       (H16  =',H16:4,' ( ',PERCENTS(H16,Baud9600):5:2,'%)');
 If HST <> 0 then
    WriteLn(OutF,'                       (HST  =',HST:4,' ( ',PERCENTS(HST,Baud9600):5:2,'%)');
 If PEP <> 0 then
    WriteLn(OutF,'                       (PEP  =',PEP:4,' ( ',PERCENTS(PEP,Baud9600):5:2,'%)');
 If MAX <> 0 then
    WriteLn(OutF,'                       (MAX  =',MAX:4,' ( ',PERCENTS(MAX,Baud9600):5:2,'%)');
 If V32 <> 0 then
    WriteLn(OutF,'                       (V32  =',V32:4,' ( ',PERCENTS(V32,Baud9600):5:2,'%)');
 If V32B <> 0 then
    WriteLn(OutF,'                       (V32B =',V32B:4,' ( ',PERCENTS(V32B,Baud9600):5:2,'%)');
 If V32T <> 0 then
    WriteLn(OutF,'                       (V32T =',V32T:4,' ( ',PERCENTS(V32T,Baud9600):5:2,'%)');
 If V34 <> 0 then
    WriteLn(OutF,'                       (V34  =',V34:4,' ( ',PERCENTS(V34,Baud9600):5:2,'%)');
 If V42 <> 0 then
    WriteLn(OutF,'                       (V42  =',V42:4,' ( ',PERCENTS(V42,Baud9600):5:2,'%)');
 If V42B <> 0 then
    WriteLn(OutF,'                       (V42B =',V42B:4,' ( ',PERCENTS(V42B,Baud9600):5:2,'%)');
 If Baud2400 <> 0 then
    WriteLn(OutF,'           2400 = ',Baud2400:4,' (', PERCENTS(Baud2400,i):5:2,'%)');
 If Baud1200 <> 0 then
    WriteLn(OutF,'           1200 = ',Baud1200:4,' (', PERCENTS(Baud1200,i):5:2,'%)');
 If 300 <> 0 then
    WriteLn(OutF,'            300 = ',Baud300:4,' (', PERCENTS(Baud300,i):5:2,'%)');
 If ISDN <> 0 then
    WriteLn(OutF,'           ISDN = ',ISDN:4,' (', PERCENTS(ISDN,i):5:2,'%)');
 WriteLn(OutF);
 WriteLn(OutF,'     File Request Statuses (zero omitted)');
 WriteLn(OutF,'     =====================');
 If XA <> 0 then
    WriteLn(OutF,'             XA = ',XA:4,' (', PERCENTS(XA,i):5:2,'%)');
 If XB <> 0 then
    WriteLn(OutF,'             XB = ',XB:4,' (', PERCENTS(XB,i):5:2,'%)');
 If XC <> 0 then
    WriteLn(OutF,'             XC = ',XC:4,' (', PERCENTS(XC,i):5:2,'%)');
 If XP <> 0 then
    WriteLn(OutF,'             XP = ',XP:4,' (', PERCENTS(XP,i):5:2,'%)');
 If XR <> 0 then
    WriteLn(OutF,'             XR = ',XR:4,' (', PERCENTS(XR,i):5:2,'%)');
 If XW <> 0 then
    WriteLn(OutF,'             XW = ',XW:4,' (', PERCENTS(XW,i):5:2,'%)');
 If XX <> 0 then
    WriteLn(OutF,'             XX = ',XX:4,' (', PERCENTS(XX,i):5:2,'%)');
 WriteLn(OutF);
 WriteLn(OutF,'      IP             (zero omitted)');
 WRiteLn(OutF,'     Flag      Protocol          Systems');
 WriteLn(OutF,'     =====     ========          =======');
 WriteLn(OutF,'     IBN       Binkp            ',IBN:4,' (',PERCENTS(IBN,i):5:2,'%)');
 WriteLn(OutF,'     IFC       Raw ifcico       ',IFC:4,' (',PERCENTS(IFC,i):5:2,'%)');
 WriteLn(OutF,'     IFT       FTP              ',IFT:4,' (',PERCENTS(IFT,i):5:2,'%)');
 WriteLn(OutF,'     ITN       Telnet           ',ITN:4,' (',PERCENTS(ITN,i):5:2,'%)');
 WriteLn(OutF,'     IVM       Vmodem           ',IVM:4,' (',PERCENTS(IVM,i):5:2,'%)');
 WriteLn(OutF,'     INO4      IPv6 only        ',INO4:4,' (',PERCENTS(INO4,i):5:2,'%)');
 WriteLn(OutF,'     IP        Others           ',IP:4,' (',PERCENTS(IP,i):5:2,'%)');
 WriteLn(OutF);
 WriteLn(OutF,' Report produced by Netstats - A P.D and Open Source program');
 WriteLn(OutF,'   Updated by Vincent Coen and available from 2:250/1 ]');
 WriteLn(OutF,' and at http://www.applewood.linkpc.net/files/NetStats ');
 WriteLn(OutF);
 WriteLn(OutF,'        ', Version);
 Flush (OutF);
 Close (OutF);
End;
{--------------------------------------------------------------------}

Procedure GetNewList(Var ListName:PathStr);
Var ListDir  : DirStr;
    ListFile : NameStr;
    ListExt  : ExtStr;
    SearchData : SearchRec;
    CheckUse   : longInt;
    CheckName  : String;

Begin
  ListName := FExpand(ListName);
  FSplit(ListName,ListDir,ListFile,ListExt);
  ListName := ListDir + ListFile + '.*';
  FindFirst(ListName,Archive,SearchData);
  If DOSERROR <> 0 then
  Begin
    Writeln('OOPS!  The Selected NodeList file does not exist, aborting.');
    Halt(1);
  End;
  CheckUse := 0;
  WriteLn(' Getting the latest Nodelist File for you....');
  Repeat
    If (SearchData.Time > CheckUse)
                AND
    (SearchData.Name[Length(SearchData.Name)] in ['1','2','3','4','5','6','7','8','9','0'])    then
      Begin
        CheckUse := SearchData.Time;
        UnPackTime(CheckUse,FTIME);
        With FTIME
          Do
            begin
              FYear  := FTIME.Year;
              FMonth := FTIME.Month;
              FDay   := FTIME.Day;
            end;
        Fkb    := SearchData.Size/1024;
        CheckName := SearchData.Name;
        WriteLn('File Match Found: ',CheckName,'  ',FYear,'/',Fmonth,'/',Fday,' ',FKb:7:1,' KBytes.');
      End;
    FindNext(SearchData);
  Until DOSERROR = 18;
  ListName := ListDir + CheckName;
End;

{--------------------------------------------------------------------}

FUNCTION Exist(fname:string): Boolean;
{ Tests for existence of a file - will match wildcards, too }
Var
   Result : Boolean;
   DirInfo: SearchRec;
   code   : integer;

begin { this function }
     Result := False;
     FindFirst(fname,AnyFile,DirInfo);
     code := DosError;
     case code of
          0:    begin
                  Result := True;
                end;
          2,3:  begin
                  Result := False;
                 { WriteLn(' No Such File/Path/Directory Found'); }
                end;
         18:    begin
                  Result := False;
                end;
     else
         begin
              Result := False;
              WriteLn;
              WriteLn(' Undefined error code ',code,' has been encountered.');
         end;
     end;
{ writeln(code); }
Exist := Result;
End; { of Function EXIST }
{ ----------------------------------------------------------------- }

PROCEDURE ItsTime(Fname:String);
Var
   DirInfo: SearchRec;
   code   : integer;
begin { this function }
     FindFirst(fname,AnyFile,DirInfo);
     code := DosError;
     case code of
          0:    begin
                  UnPackTime(DirInfo.Time,FTIME);
                  With FTIME
                    Do
                      begin
                        FYear  := FTIME.Year;
                        FMonth := FTIME.Month;
                        FDay   := FTIME.Day;
                      end;
                        Fkb    := DirInfo.Size/1024;
                end;
          2,3:  begin
                  WriteLn(' No Such File/Path/Directory Found');
                end;
         18:    begin
                  WriteLn(' Unusual Error Code: 18');
                end;
     else
         begin
              WriteLn;
              WriteLn(' Undefined error code ',code,' has been encountered.');
         end;
     end;

End;
{ ----------------------------------------------------------------- }

Procedure DataFile;

CONST
  Q='"';
  C=',';

Var OutFile : text;
Begin

 Assign(OutFile,'NETSTATS.DAT');

 if Exist ('NETSTATS.DAT')
 then
   Append(OutFile)
 else
   Rewrite(OutFile);

   Write(OutFile,

   FKb:5:1,C,
   FYear,C,
   FMonth,C,
   FDay,C,
   Q,IfnaList,Q,C,
   i,C,
   NonComments,C,

   ZoneCount,C,
   RegionCount,C,
   HostCount,C,
   HubCount,C,
   DownCount,C,
   PvtCount,C,
   HoldCount,C,
   AdminOVHD,C,
   ActivSysops,C,

   Baud33600,C,
   Baud9600,C,
   Baud2400,C,
   Baud1200,C,
   Baud300,C,

   H16,C,
   HST,C,
   PEP,C,
   MAX,C,
   V32,C,
   V32B,C,
   V32T,C,
   V34,C,
   V42,C,
   V42B,C,
   ISDN,C,

   IBN,C,
   IFC,C,
   IFT,C,
   ITN,C,
   IVM,C,
   IP,C,
   INO4,C,

   XA,C,
   XB,C,
   XC,C,
   XP,C,
   XR,C,
   XW,C,
   XX,C,

   CrashMail,C,
   MailOnly );
   WriteLn(OutFile);

   Flush(OutFile);
   Close(OutFile);
   Writeln;
   WriteLn(' Output data also as NETSTATS.DAT in comma-delimited format.')
end;

{--------------------------------------------------------------------}

BEGIN { Main Program Starts here }

ClrScr;

WriteLn('                   ',Version);
WriteLn;
writeln( ' This program is placed into the Public Domain as Source code and is');
writeln( ' available by file requesting netstu*.zip from FidoNet 2:250/1 ');
writeln( ' or via vbcoen@gmail.com ');
WriteLn;
WriteLn( ' The latest version can also be found at :');
WriteLn( ' http://www.applewood.linkpc.net/files/NetStats');
WriteLn;
WriteLn( ' This area always holds the latest version.');
Writeln;
writeln( ' Vincent Coen, Sysop at 2:250/1');
writeln( '----------------------------------------------------------------');

TextColor(7);
TextBackground(0);
LowVideo;

   ParmCount := ParamCount;
   Case ParmCount of
     0:
       begin
            Writeln( ' You may enter the NODELIST file as');
            WriteLn( ' command-line parameters. Examples:');
            WriteLn('   netstats c:\opus\nodelist\nodelist for Dos, Windows');
            WriteLn('       or ');
            WriteLn('   netstats /home/mbse/var/nodelists/nodelist for Linux');
            WriteLn;
            WriteLn(' Uses the nodelist latest nodelist as input from the');
            WriteLn(' specified subdirectory and checks for duplicates in');
            WriteLn(' that nodelist. The output file is ALWAYS called  NetStats.rpt');
            WriteLn;
            WriteLn(' Since you didn''t give a parameter, you will now');
            WriteLn(' be prompted for the Nodelist name..... ');
            Goto GetFileNames;
       end;
     1:
       begin
            IfnaList := ParamStr(1);
            GetNewList(IfnaList);
            Goto GotFileNames;
       end;
     Else
       begin
            WriteLn(' Error in number of command line parameters');
            WriteLn(' Halting program....');
            Halt(3);
       end;
     End;

GetFileNames:
   WriteLn;
   WriteLn(' What is the Nodelist file to be used ? ');
   ReadLn(IfnaList);
GotFileNames:
   Writeln;
   WriteLn(' (Using ', IfnaList, ' as Input file)');
   WriteLn(' (Using NetStats.rpt as reporting file)');

   { Open the IFNA nodelist file }
      If EXIST(IfnaList)
      then
        begin
          Assign(INodeList, IfnaList);
          ItsTime(IfnaList);
        end
      else
        Begin
          WriteLn(' The requested file: ',IfnaList, ' does not exist!');
          Halt(2);
        End;
      SetTextBuf(INodeList,Buf);  { reads that sucker 20K at a time }
      Reset(INodeList);
      writeln('  ',IfnaList,' Open''d for analysis ...');

      i := 0; { init the counter }
      Zone        := i;        { current Zone }
      Z1Down      := i;
      Z2Down      := i;
      Z3Down      := i;
      Z4Down      := i;
      Z1Hold      := i;
      Z2Hold      := i;
      Z3Hold      := i;
      Z4Hold      := i;
      Z1Pvt       := i;
      Z2Pvt       := i;
      Z3Pvt       := i;
      Z4Pvt       := i;
      Z1Sysops    := i;
      Z2Sysops    := i;
      Z3Sysops    := i;
      Z4Sysops    := i;
      Z1Region    := i;
      Z2Region    := i;
      Z3Region    := i;
      Z4Region    := i;
      Z1Host      := i;
      Z2Host      := i;
      Z3Host      := i;
      Z4Host      := i;
      Z1Hub       := i;
      Z2Hub       := i;
      Z3Hub       := i;
      Z4Hub       := i;

      ZoneCount    := i;
      RegionCount  := i;
      HostCount    := i;
      HubCount     := i;
      PvtCount     := i;
      DownCount    := i;
      HoldCount    := i;

      Baud33600    := i;

      Baud9600     := i;
      Baud2400     := i;
      Baud1200     := i;
      Baud300      := i;

      ISDN         := i;

      CrashMail    := i;
      MailOnly     := i;
      HST          := i;
      PEP          := i;
      H16          := i;
      MAX          := i;
      V32          := i;
      V32b         := i;
      V32T         := i;
      V34          := i;
      V42          := i;
      V42b         := i;
      NonComments  := i;

      IBN          := i;   { more Flags }
      IFC          := i;
      IFT          := i;
      ITN          := i;
      IVM          := i;
      IP           := i;
      INO4         := i;
      XA           := i;
      XB           := i;
      XC           := i;
      XP           := i;
      XR           := i;
      XW           := i;
      XX           := i;

      Z1ZoneName := '';
      Z2ZoneName := '';
      Z3ZoneName := '';
      Z4ZoneName := '';

      While NOT Eof(INodeList) do
      begin
         ReadLn(INodelist,Aline); { get a node from FidoNet List }
         ParseLines(Aline,Lines,AComment);

         If NOT AComment then
         Begin
           Inc(NonComments);
           If Lines.Kind = 'Zone' then
           begin
              Val(Lines.Number,NetNumber,Code);
              WriteLn;
              WriteLn;
              HighVideo;
              TextColor(12);
              WriteLn('Zone ',NetNumber,'  (',Lines.BbsName,')');  {   Lines.City_ST,')');  }
              TextColor(7);
              LowVideo;
              Inc(ZoneCount);
              Zone := NetNumber;
              if  Zone = 1 then Z1ZoneName := Lines.BBsName;
              if  Zone = 2 then Z2ZoneName := Lines.BBsName;
              if  Zone = 3 then Z3ZoneName := Lines.BBsName;
              if  Zone = 4 then Z4ZoneName := Lines.BBsName;
           end;

           If Lines.Kind = 'Region' then
           begin
              Val(Lines.Number,NetNumber,Code);
              TextColor(14);
              WriteLn;
              WriteLn('Region ',NetNumber,'  (',Lines.BbsName,')');  { Lines.City_ST,')');  }
              TextColor(7);
              Inc(RegionCount);
              if  Zone = 1 then
                 inc(Z1Region);
              if  Zone = 2 then
                 inc(Z2Region);
              if  Zone = 3 then
                  inc(Z3Region);
              if  Zone = 4 then
                  inc(Z4Region);
           end;

           If Lines.Kind = 'Host' then
           begin
              Val(Lines.Number,NetNumber,Code);
              If WhereX > 70 then WriteLn;
              TextColor(9);
              Write( ' Net ',NetNumber);
              TextColor(7);
              Inc(HostCount);
              if  Zone = 1 then inc(Z1Host);
              if  Zone = 2 then inc(Z2Host);
              if  Zone = 3 then inc(Z3Host);
              if  Zone = 4 then inc(Z4Host);
           end;

           If Lines.Kind = 'Hub' then
           begin
              Inc(HubCount);
              if  Zone = 1 then inc(Z1Hub);
              if  Zone = 2 then inc(Z2Hub);
              if  Zone = 3 then inc(Z3Hub);
              if  Zone = 4 then inc(Z4Hub);
           end;

           If Lines.Kind = 'Down' then
           begin
              Inc(DownCount);
              if  Zone = 1 then inc(Z1Down);
              if  Zone = 2 then inc(Z2Down);
              if  Zone = 3 then inc(Z3Down);
              if  Zone = 4 then inc(Z4Down);
           end;

           If Lines.Kind = 'Pvt' then
           begin
              Inc(PvtCount);
              if  Zone = 1 then inc(Z1Pvt);
              if  Zone = 2 then inc(Z2Pvt);
              if  Zone = 3 then inc(Z3Pvt);
              if  Zone = 4 then inc(Z4Pvt);
           end;

           If Lines.Kind = 'Hold' then
           begin
              Inc(HoldCount);
              if  Zone = 1 then inc(Z1Hold);
              if  Zone = 2 then inc(Z2Hold);
              if  Zone = 3 then inc(Z3Hold);
              if  Zone = 4 then inc(Z4Hold);
           end;

           If Lines.Kind = '' then
           Begin
             Inc(i);           { Nodes }
              if  Zone = 1 then inc(Z1Sysops);
              if  Zone = 2 then inc(Z2Sysops);
              if  Zone = 3 then inc(Z3Sysops);
              if  Zone = 4 then inc(Z4Sysops);

             { WriteLn(LST,Lines.Number); }

             If (Lines.BaudRate = '9600')  then
             begin
                Inc(Baud9600);
                If (POS('HST',Lines.Flags)) > 0 then Inc(HST);
                If (POS('PEP',Lines.Flags)) > 0 then Inc(PEP);
                If (POS('H16',Lines.Flags)) > 0 then Inc(H16);
                If (POS('MAX',Lines.Flags)) > 0 then Inc(MAX);
                if (POS('ISDN',Lines.Flags)) > 0 then Inc(ISDN);
             end;

             If (Lines.BaudRate = '33600') then
                Inc(Baud33600);

             If (Lines.BaudRate = '2400') then
                Inc(Baud2400);

             If (Lines.BaudRate = '1200') then
                Inc(Baud1200);

             If (Lines.BaudRate = '300')  then
                Inc(Baud300);

             If (POS('CM',Lines.Flags)) > 0 then Inc(CrashMail);
             If (POS('MO',Lines.Flags)) > 0 then Inc(MailOnly);

             If (POS('V32',Lines.Flags)) > 0  then Inc(V32);
             If (POS('V32b',Lines.Flags)) > 0 then Inc(V32b);
             If (POS('V32T',Lines.Flags)) > 0 then Inc(V32T);
             If (POS('V34',Lines.Flags)) > 0  then Inc(V34);
             If (POS('V42',Lines.Flags)) > 0  then Inc(V42);
             If (POS('V42b',Lines.Flags)) > 0 then Inc(V42b);

             If (POS('IBN',Lines.Flags)) > 0  then Inc(IBN);
             If (POS('IFC',Lines.Flags)) > 0  then Inc(IFC);
             If (POS('IFT',Lines.Flags)) > 0  then Inc(IFT);
             If (POS('ITN',Lines.Flags)) > 0  then Inc(ITN);
             If (POS('IVM',Lines.Flags)) > 0  then Inc(IVM);
             If (POS('IP',Lines.Flags)) > 0   then Inc(IP);
             If (POS('INO4',Lines.Flags)) > 0 then Inc(INO4);

             If (POS('XA',Lines.Flags)) > 0  then Inc(XA);
             If (POS('XB',Lines.Flags)) > 0  then Inc(XB);
             If (POS('XC',Lines.Flags)) > 0  then Inc(XC);
             If (POS('XP',Lines.Flags)) > 0  then Inc(XP);
             If (POS('XR',Lines.Flags)) > 0  then Inc(XR);
             If (POS('XW',Lines.Flags)) > 0  then Inc(XW);
             If (POS('XX',Lines.Flags)) > 0  then Inc(XX);

           End;
         End;
      end;
 {     IfnaNum := i;  }
      AdminOVHD := ZoneCount+RegionCount+HostCount+HubCount;
      Close(INodeList);

      V32 := V32 - V32b;  { back out the bis' }
      V42 := V42 - V42b;  { back out the bis' }

 TextColor(7);
 TextBackground(0);
 NormVideo;

 { Output Stats to a file }

   Show_Em('NetStats.rpt');

 { Uncomment the following for Stats to CRT output }
{
   Clrscr;
   Show_Em('');
}

 { Uncomment the following for printed Stats output }
{
   Show_Em('PRN');
}

 DataFile;

END.  { Main Program ends here   }
